import { Beatmap, BeatmapAttributes, BeatmapCompact } from "./beatmap";
import { KudosuHistory, User, UserCompact } from "./user";
import { Leader, Match, MatchInfo, MultiplayerScore, PlaylistItem, Room } from "./multiplayer";
import { GameModes, Mod } from "./misc";
import { BeatmapUserScore, Score } from "./score";
import { Rankings, Spotlight } from "./ranking";
export { Beatmap, BeatmapCompact };
export { User, UserCompact, KudosuHistory };
export { BeatmapUserScore, Score };
export { Room, Leader, PlaylistItem, MultiplayerScore };
export { GameModes };
export declare class APIError {
    message: string;
    /**
     * @param message The reason why things didn't go as expected
     */
    constructor(message: string);
}
/**
 * You can create an API instance using its `createAsync` function!
 */
export declare class API {
    client: {
        id: number;
        secret: string;
    };
    token_type: string;
    expires: Date;
    access_token: string;
    refresh_token?: string;
    user?: number;
    scopes: Scope[];
    /**
     * Use `createAsync` instead of the default constructor if you don't have at least an access_token
     */
    constructor(client?: {
        id: number;
        secret: string;
    }, token_type?: string, expires?: Date, access_token?: string, scopes?: Scope[], refresh_token?: string, user?: number);
    /**
     * The normal way to create an API instance! Make sure to `await` it
     * @param client The ID and the secret of your client, can be found on https://osu.ppy.sh/home/account/edit#new-oauth-application
     * @param user If the instance is supposed to represent a user, use their code and the redirect_uri of your application!
     * @returns A promise with an API instance (or with null if something goes wrong)
     */
    static createAsync: (client: {
        id: number;
        secret: string;
    }, user?: {
        code: string;
        redirect_uri: string;
    }) => Promise<API | null>;
    refreshToken(): Promise<boolean>;
    /**
     * @param endpoint What comes in the URL after `api/`
     * @param parameters The things to specify in the request, such as the beatmap_id when looking for a beatmap
     * @param number_try How many attempts there's been to get the data
     * @returns A Promise with either the API's response or `false` upon failing
     */
    private request;
    /**
     * REQUIRES A USER ASSOCIATED TO THE API OBJECT (PUBLIC SCOPE)
     */
    getResourceOwner(gamemode?: GameModes): Promise<User | APIError>;
    getUser(user: {
        id?: number;
        username?: string;
    } | UserCompact, gamemode?: GameModes): Promise<User | APIError>;
    getUsers(ids?: number[]): Promise<UserCompact[] | APIError>;
    getUserScores(limit: number, user: {
        id: number;
    } | UserCompact, type: "best" | "firsts" | "recent", options?: {
        gamemode?: GameModes;
        include_fails?: Boolean;
        offset?: number;
    }): Promise<Score[] | APIError>;
    getUserKudosu(user: {
        id: number;
    } | UserCompact, limit?: number, offset?: number): Promise<KudosuHistory[] | APIError>;
    /**
     * REQUIRES A USER ASSOCIATED TO THE API OBJECT (FRIENDS.READ SCOPE)
     */
    getFriends(): Promise<UserCompact[] | APIError>;
    getBeatmap(beatmap: {
        id: number;
    } | BeatmapCompact): Promise<Beatmap | APIError>;
    getBeatmaps(ids?: number[]): Promise<Beatmap[] | APIError>;
    /**
     * @remarks Will ignore the settings of your mods
     */
    getBeatmapAttributes(beatmap: {
        id: number;
    } | BeatmapCompact, gamemode: GameModes, mods: Mod[] | string[] | number): Promise<BeatmapAttributes | APIError>;
    getBeatmapUserScore(beatmap: {
        id: number;
    } | BeatmapCompact, user: {
        id: number;
    } | UserCompact, gamemode?: GameModes): Promise<BeatmapUserScore | APIError>;
    getRoom(room: {
        id: number;
    } | Room): Promise<Room | APIError>;
    getMatch(id: number): Promise<Match | APIError>;
    getMatches(): Promise<MatchInfo[] | APIError>;
    /**
     * REQUIRES A USER ASSOCIATED TO THE API OBJECT (PUBLIC SCOPE)
     */
    getRooms(mode?: "owned" | "participated" | "ended"): Promise<Room[] | APIError>;
    /**
     * REQUIRES A USER ASSOCIATED TO THE API OBJECT (PUBLIC SCOPE)
     */
    getRoomLeaderboard(room: {
        id: number;
    } | Room): Promise<Leader[] | APIError>;
    /**
     * REQUIRES A USER ASSOCIATED TO THE API OBJECT (PUBLIC SCOPE)
     */
    getPlaylistItemScores(item: {
        id: number;
        room_id: number;
    } | PlaylistItem): Promise<MultiplayerScore[] | APIError>;
    getRanking(gamemode: GameModes, type: "charts" | "country" | "performance" | "score", filter: "all" | "friends", options?: {
        country?: number;
        spotlight?: {
            id: number;
        } | Spotlight;
        variant?: "4k" | "7k";
    }): Promise<Rankings | APIError>;
    getSpotlights(): Promise<Spotlight[] | APIError>;
}
type Scope = "chat.write" | "delegate" | "forum.write" | "friends.read" | "identify" | "public";
export declare function generateAuthorizationURL(client_id: number, redirect_uri: string, scopes: Scope[]): string;
