"use strict";
var _a;
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateAuthorizationURL = exports.API = exports.APIError = exports.GameModes = void 0;
const axios_1 = require("axios");
const misc_1 = require("./misc");
Object.defineProperty(exports, "GameModes", { enumerable: true, get: function () { return misc_1.GameModes; } });
class APIError {
    /**
     * @param message The reason why things didn't go as expected
     */
    constructor(message) {
        this.message = message;
    }
}
exports.APIError = APIError;
/**
 * You can create an API instance using its `createAsync` function!
 */
class API {
    // vvv Whole token handling thingie is down there vvv
    /**
     * Use `createAsync` instead of the default constructor if you don't have at least an access_token
     */
    constructor(client, token_type, expires, access_token, scopes, refresh_token, user) {
        this.client = client !== null && client !== void 0 ? client : { id: 0, secret: "" };
        this.token_type = token_type !== null && token_type !== void 0 ? token_type : "";
        this.expires = expires !== null && expires !== void 0 ? expires : new Date();
        this.access_token = access_token !== null && access_token !== void 0 ? access_token : "";
        this.scopes = scopes !== null && scopes !== void 0 ? scopes : [];
        if (refresh_token) {
            this.refresh_token = refresh_token;
        }
        if (user) {
            this.user = user;
        }
    }
    async refreshToken() {
        if (!this.refresh_token) {
            return false;
        }
        let data = {
            client_id: this.client.id,
            client_secret: this.client.secret,
            grant_type: "refresh_token",
            refresh_token: this.refresh_token
        };
        let response = await axios_1.default.post(`https://osu.ppy.sh/oauth/token`, data, { headers: {
                "Accept": "application/json",
                "Content-Type": "application/json"
            } }).catch((e) => console.log("osu!api v2 ->", `(${e.name}) ${e.message}`));
        if (response && response.data && response.data.token_type) {
            let date = new Date();
            date.setSeconds(date.getSeconds() + response.data.expires_in);
            this.token_type = response.data.token_type;
            this.expires = date;
            this.access_token = response.data.access_token;
            if (response.data.refresh_token) {
                this.refresh_token = response.data.refresh_token;
            }
        }
        else {
            return false;
        }
        return true;
    }
    // ^^^ Whole token handling thingie is up there ^^^
    /**
     * @param endpoint What comes in the URL after `api/`
     * @param parameters The things to specify in the request, such as the beatmap_id when looking for a beatmap
     * @param number_try How many attempts there's been to get the data
     * @returns A Promise with either the API's response or `false` upon failing
     */
    async request(method, endpoint, parameters, number_try) {
        const max_tries = 5;
        if (!number_try) {
            number_try = 1;
        }
        let to_retry = false;
        let data = {};
        let params = {};
        if (parameters !== undefined) {
            for (let i = 0; i < Object.entries(parameters).length; i++) {
                if (!String(Object.values(parameters)[i]).length || Object.values(parameters)[i] === undefined) {
                    i--;
                    delete parameters[Object.keys(parameters)[i + 1]];
                }
            }
            if (method === "post") {
                data = parameters !== null && parameters !== void 0 ? parameters : {};
            }
            else {
                params = parameters !== null && parameters !== void 0 ? parameters : {};
            }
        }
        let request = {
            method,
            baseURL: "https://osu.ppy.sh/api/v2/",
            url: endpoint,
            headers: {
                "Accept": "application/json",
                "Accept-Encoding": "gzip",
                "Content-Type": "application/json",
                "User-Agent": "osu-api-v2-js (https://github.com/TTTaevas/osu-api-v2-js)",
                "Authorization": `${this.token_type} ${this.access_token}`
            },
            data,
            params
        };
        const resp = await (0, axios_1.default)(request)
            .catch((error) => {
            if (axios_1.default.isAxiosError(error)) {
                if (error.response) {
                    console.log("osu!api v2 ->", error.response.statusText, error.response.status, { type: endpoint, parameters });
                    if (error.response.status === 401)
                        console.log("osu!api v2 -> Server responded with status code 401, maybe you need to do this action as an user?");
                    if (error.response.status === 429) {
                        console.log("osu!api v2 -> Server responded with status code 429, you're sending too many requests at once and are getting rate-limited!");
                        if (number_try !== undefined && number_try < max_tries) {
                            console.log(`osu!api v2 -> Will request again in a few instants... (Try #${number_try})`);
                        }
                        to_retry = true;
                    }
                }
                else if (error.request) {
                    console.log("osu!api v2 ->", "Request made but server did not respond", `(Try #${number_try})`, { type: endpoint, parameters });
                    to_retry = true;
                }
                else { // Something happened in setting up the request that triggered an error, I think
                    console.error(error);
                }
            }
            else {
                console.error(error);
            }
        });
        if (resp) {
            console.log("osu!api v2 ->", resp.statusText, resp.status, { type: endpoint, parameters });
            return resp.data;
        }
        else {
            /**
             * Under specific circumstances, we want to retry our request automatically
             * However, spamming the server during the same second in any of these circumstances would be pointless
             * So we wait for 1 to 5 seconds to make our request, 5 times maximum
             */
            if (to_retry && number_try < max_tries) {
                let to_wait = (Math.floor(Math.random() * (500 - 100 + 1)) + 100) * 10;
                await new Promise(res => setTimeout(res, to_wait));
                return await this.request(method, endpoint, parameters, number_try + 1);
            }
            else {
                return false;
            }
        }
    }
    // USER STUFF
    /**
     * REQUIRES A USER ASSOCIATED TO THE API OBJECT (PUBLIC SCOPE)
     */
    async getResourceOwner(gamemode) {
        let response = await this.request("get", "me", { mode: gamemode !== undefined ? misc_1.GameModes[gamemode] : "" });
        if (!response) {
            return new APIError(`No User could be found`);
        }
        return correctType(response);
    }
    async getUser(user, gamemode) {
        if (!user.id && !user.username) {
            return new APIError("No proper `user` argument was given");
        }
        let key = user.id !== undefined ? "id" : "username";
        let lookup = user.id !== undefined ? user.id : user.username;
        let response = await this.request("get", `users/${lookup}${gamemode !== undefined ? `/${misc_1.GameModes[gamemode]}` : ""}`, { key });
        if (!response) {
            return new APIError(`No User could be found (user id: ${user.id} / username: ${user.username})`);
        }
        return correctType(response);
    }
    async getUsers(ids) {
        let response = await this.request("get", "users", { ids });
        if (!response || !response.users || !response.users.length) {
            return new APIError(`No User could be found (ids: ${ids})`);
        }
        return response.users.map((u) => correctType(u));
    }
    async getUserScores(limit, user, type, options) {
        let mode = options && options.gamemode !== undefined ? misc_1.GameModes[options.gamemode] : "";
        let offset = options && options.offset !== undefined ? options.offset : "";
        let include_fails = options && options.include_fails !== undefined ? options.include_fails : "";
        let response = await this.request("get", `users/${user.id}/scores/${type}`, { limit, mode, offset, include_fails });
        if (!response || !response.length) {
            return new APIError(`No Score could be found (id: ${user.id} / type: ${type})`);
        }
        return response.map((s) => correctType(s));
    }
    async getUserKudosu(user, limit, offset) {
        let response = await this.request("get", `users/${user.id}/kudosu`, { limit, offset });
        if (!response || !response.length) {
            return new APIError(`No Kudosu could be found (id: ${user.id})`);
        }
        return response.map((k) => correctType(k));
    }
    /**
     * REQUIRES A USER ASSOCIATED TO THE API OBJECT (FRIENDS.READ SCOPE)
     */
    async getFriends() {
        let response = await this.request("get", "friends");
        if (!response || !response.length) {
            return new APIError(`No Friend could be found`);
        }
        return response.map((f) => correctType(f));
    }
    // BEATMAP STUFF
    async getBeatmap(beatmap) {
        let response = await this.request("get", `beatmaps/${beatmap.id}`);
        if (!response) {
            return new APIError(`No Beatmap could be found (id: ${beatmap.id})`);
        }
        return correctType(response);
    }
    async getBeatmaps(ids) {
        let response = await this.request("get", "beatmaps", { ids });
        if (!response || !response.beatmaps || !response.beatmaps.length) {
            return new APIError(`No Beatmap could be found (ids: ${ids})`);
        }
        return response.beatmaps.map((b) => correctType(b));
    }
    /**
     * @remarks Will ignore the settings of your mods
     */
    async getBeatmapAttributes(beatmap, gamemode, mods) {
        let response = await this.request("post", `beatmaps/${beatmap.id}/attributes`, { ruleset_id: gamemode, mods });
        if (!response) {
            return new APIError(`No Beatmap could be found (id: ${beatmap.id})`);
        }
        return correctType(response);
    }
    async getBeatmapUserScore(beatmap, user, gamemode) {
        let response = await this.request("get", `beatmaps/${beatmap.id}/scores/users/${user.id}`);
        if (!response) {
            return new APIError(`No Score could be found (beatmap: ${beatmap.id} / user: ${user.id})`);
        }
        return correctType(response);
    }
    // MULTIPLAYER STUFF
    async getRoom(room) {
        let response = await this.request("get", `rooms/${room.id}`);
        if (!response) {
            return new APIError(`No Room could be found (id: ${room.id})`);
        }
        return correctType(response);
    }
    async getMatch(id) {
        let response = await this.request("get", `matches/${id}`);
        if (!response) {
            return new APIError(`No Match could be found (id: ${id})`);
        }
        return correctType(response);
    }
    async getMatches() {
        let response = await this.request("get", "matches");
        if (!response || !response.matches || !response.matches.length) {
            return new APIError(`No Match could be found`);
        }
        return response.matches.map((m) => correctType(m));
    }
    /**
     * REQUIRES A USER ASSOCIATED TO THE API OBJECT (PUBLIC SCOPE)
     */
    async getRooms(mode) {
        let response = await this.request("get", `rooms/${mode ? mode : ""}`);
        if (!response || !response.length) {
            {
                return new APIError(`No Room could be found${mode ? ` (mode: ${mode})` : ""}`);
            }
        }
        return response.map((r) => correctType(r));
    }
    /**
     * REQUIRES A USER ASSOCIATED TO THE API OBJECT (PUBLIC SCOPE)
     */
    async getRoomLeaderboard(room) {
        let response = await this.request("get", `rooms/${room.id}/leaderboard`);
        if (!response || !response.leaderboard || !response.leaderboard.length) {
            return new APIError(`No Leaderboard could be found (id: ${room.id})`);
        }
        return response.leaderboard.map((l) => correctType(l));
    }
    /**
     * REQUIRES A USER ASSOCIATED TO THE API OBJECT (PUBLIC SCOPE)
     */
    async getPlaylistItemScores(item) {
        let response = await this.request("get", `rooms/${item.room_id}/playlist/${item.id}/scores`);
        if (!response || !response.scores || !response.scores.length) {
            return new APIError(`No Item could be found (room: ${item.room_id} / item: ${item.id})`);
        }
        return response.scores.map((s) => correctType(s));
    }
    // RANKING STUFF
    async getRanking(gamemode, type, filter, options) {
        let response = await this.request("get", `rankings/${misc_1.GameModes[gamemode]}/${type}`, {
            filter,
            country: options === null || options === void 0 ? void 0 : options.country,
            spotlight: options === null || options === void 0 ? void 0 : options.spotlight,
            variant: options === null || options === void 0 ? void 0 : options.variant
        });
        if (!response) {
            return new APIError(`No Ranking could be found (type: ${type})`);
        }
        return correctType(response);
    }
    async getSpotlights() {
        let response = await this.request("get", "spotlights");
        if (!response || !response.spotlights || !response.spotlights.length) {
            return new APIError(`No Spotlight could be found`);
        }
        return response.spotlights.map((s) => correctType(s));
    }
}
exports.API = API;
_a = API;
/**
 * The normal way to create an API instance! Make sure to `await` it
 * @param client The ID and the secret of your client, can be found on https://osu.ppy.sh/home/account/edit#new-oauth-application
 * @param user If the instance is supposed to represent a user, use their code and the redirect_uri of your application!
 * @returns A promise with an API instance (or with null if something goes wrong)
 */
API.createAsync = async (client, user) => {
    const me = new API();
    me.client = client;
    let data = {
        client_id: client.id,
        client_secret: client.secret,
        grant_type: user ? "authorization_code" : "client_credentials"
    };
    if (user) {
        data.redirect_uri = user.redirect_uri;
        data.code = user.code;
    }
    else {
        data.scope = "public";
    }
    let response = await axios_1.default.post(`https://osu.ppy.sh/oauth/token`, data, { headers: {
            "Accept": "application/json",
            "Content-Type": "application/json"
        } }).catch((e) => console.log("osu!api v2 ->", `(${e.name}) ${e.message}`));
    if (response && response.data && response.data.access_token) {
        let token = response.data.access_token;
        let date = new Date();
        date.setSeconds(date.getSeconds() + response.data.expires_in);
        me.token_type = response.data.token_type;
        me.expires = date;
        me.access_token = token;
        if (response.data.refresh_token) {
            me.refresh_token = response.data.refresh_token;
        }
        let token_payload = JSON.parse(Buffer.from(token.substring(token.indexOf(".") + 1, token.lastIndexOf(".")), "base64").toString('ascii'));
        if (token_payload.sub && token_payload.sub.length) {
            me.user = Number(token_payload.sub);
        }
        me.scopes = token_payload.scopes;
    }
    else {
        return null;
    }
    return me;
};
function generateAuthorizationURL(client_id, redirect_uri, scopes) {
    let s = String(scopes).replace(/,/g, "%20");
    return `https://osu.ppy.sh/oauth/authorize?client_id=${client_id}&redirect_uri=${redirect_uri}&scope=${s}&response_type=code`;
}
exports.generateAuthorizationURL = generateAuthorizationURL;
/**
 * Some stuff doesn't have the right type to begin with, such as dates, which are being returned as strings, this fixes that
 * @param x Anything, but should be a string, an array that contains a string, or an object which has a string
 * @returns x, but with it (or what it contains) now having the correct type
 */
function correctType(x) {
    if (typeof x === "boolean") {
        return x;
    }
    else if (!isNaN(x)) {
        return x === null ? null : Number(x);
    }
    else if (/^[+-[0-9][0-9]+-[0-9]{2}-[0-9]{2}T[0-9]{2}:[0-9]{2}:[0-9]{2}Z$/.test(x) ||
        /^[+-[0-9][0-9]+-[0-9]{2}-[0-9]{2}$/g.test(x)) {
        return new Date(x);
    }
    else if (/^[+-[0-9][0-9]+-[0-9]{2}-[0-9]{2} [0-9]{2}:[0-9]{2}:[0-9]{2}$/.test(x)) {
        x += "Z";
        return new Date(x);
    }
    else if (/^[+-[0-9][0-9]+-[0-9]{2}-[0-9]{2}T[0-9]{2}:[0-9]{2}:[0-9]{2}\+[0-9]{2}:[0-9]{2}$/.test(x)) {
        x = x.substring(0, x.indexOf("+")) + "Z";
        return new Date(x);
    }
    else if (Array.isArray(x)) {
        return x.map((e) => correctType(e));
    }
    else if (typeof x === "object" && x !== null) {
        const k = Object.keys(x);
        const v = Object.values(x);
        for (let i = 0; i < k.length; i++) {
            x[k[i]] = correctType(v[i]);
        }
    }
    return x;
}
